// Copyright (C) 1998-1999 Scott Cutler
// Please see the "readme.txt" file for license details

#include "include.h"


/* Important */
FX_ENTRY void FX_CALL grBufferSwap( int swap_interval ) {
	vlog("Called grBufferSwap");

	_lpD3DD->EndScene();
	_lpD3DFrame->ShowFrame();
	_lpD3DD->BeginScene();

}




/* Only for Quake 2 */
FX_ENTRY void FX_CALL grRenderBuffer(GrBuffer_t buffer) {
	vlog("Called grRenderBuffer");

}


/* Important */
FX_ENTRY void FX_CALL grDepthBufferFunction( GrCmpFnc_t function ) {
	vlog("Called grDepthBufferFunction: function=" << function);

//	_depthFunc = function;

	switch(function) {
	case GR_CMP_NEVER:		_lpD3DD->SetRenderState(D3DRENDERSTATE_ZFUNC, D3DCMP_NEVER);		break;
	case GR_CMP_LESS:		_lpD3DD->SetRenderState(D3DRENDERSTATE_ZFUNC, D3DCMP_LESS);			break;
	case GR_CMP_EQUAL:		_lpD3DD->SetRenderState(D3DRENDERSTATE_ZFUNC, D3DCMP_EQUAL);		break;
	case GR_CMP_LEQUAL:		_lpD3DD->SetRenderState(D3DRENDERSTATE_ZFUNC, D3DCMP_LESSEQUAL);	break;
	case GR_CMP_GREATER:	_lpD3DD->SetRenderState(D3DRENDERSTATE_ZFUNC, D3DCMP_GREATER);		break;
	case GR_CMP_NOTEQUAL:	_lpD3DD->SetRenderState(D3DRENDERSTATE_ZFUNC, D3DCMP_NOTEQUAL);		break;
	case GR_CMP_GEQUAL:		_lpD3DD->SetRenderState(D3DRENDERSTATE_ZFUNC, D3DCMP_GREATEREQUAL);	break;
	case GR_CMP_ALWAYS:		_lpD3DD->SetRenderState(D3DRENDERSTATE_ZFUNC, D3DCMP_ALWAYS);		break;
	}
}




/* Only for Marble32 */
FX_ENTRY void FX_CALL grClipWindow(FxU32 minx, FxU32 miny, FxU32 maxx, FxU32 maxy ) {
	vlog("Called grClipWindow: left=" << minx << "  right=" << maxx << "  top=" << miny << "  bottom=" << maxy);

	Voodoo.viewport.dwX = minx;
	Voodoo.viewport.dwY = miny;
	Voodoo.viewport.dwWidth = maxx - minx;
	Voodoo.viewport.dwHeight = maxy - miny;

	if (_lpD3DViewport->SetViewport2(&Voodoo.viewport) != D3D_OK) log("  Error in SetViewport2");;

}


void *temp;

/* Maybe important */
FX_ENTRY FxBool FX_CALL grLfbLock( GrLock_t type, GrBuffer_t buffer, GrLfbWriteMode_t writeMode,
           GrOriginLocation_t origin, FxBool pixelPipeline, GrLfbInfo_t *info) {

	vlog("Called grLfbLock");
	vlog("  type=" << type);
	vlog("  buffer=" << buffer);
	vlog("  writemode=" << writeMode);
	vlog("  origin=" << origin);
	vlog("  pixelpipeline=" << pixelPipeline);

	DWORD dwFlags = 0;
	DDSURFACEDESC2 DDSD;

	DDSD.dwSize = sizeof(DDSURFACEDESC2);

	if (type == GR_LFB_READ_ONLY)
		dwFlags |= DDLOCK_READONLY;
	if (type == GR_LFB_WRITE_ONLY)
		dwFlags |= DDLOCK_WRITEONLY;

	switch (buffer) {
	case GR_BUFFER_FRONTBUFFER:		_lpDDSRender = _lpD3DFrame->GetFrontBuffer(); break;
	case GR_BUFFER_BACKBUFFER:		_lpDDSRender = _lpD3DFrame->GetBackBuffer(); break;
	case GR_BUFFER_AUXBUFFER:		break;
	case GR_BUFFER_DEPTHBUFFER:		break;
	case GR_BUFFER_ALPHABUFFER:		break;
	case GR_BUFFER_TRIPLEBUFFER:	break;
	}

	if (_lpDDSRender->Lock(0, &DDSD, DDLOCK_WAIT, 0) != D3D_OK) {
		log("  Error locking surface in grLfbLock"); 
		return FXFALSE;
	}
                       
	info->size = sizeof(GrLfbInfo_t);
	info->lfbPtr = (void *)DDSD.lpSurface;
	info->strideInBytes = DDSD.lPitch;
	info->writeMode = writeMode;
	info->origin = origin;

	return FXTRUE;
}

/* Maybe important */
FX_ENTRY FxBool FX_CALL grLfbUnlock( GrLock_t type, GrBuffer_t buffer ) {
	vlog("Called grLfbUnlock");

	_lpDDSRender->Unlock(0);

	return FXTRUE;
}


FX_ENTRY FxBool FX_CALL grLfbWriteRegion( GrBuffer_t dst_buffer, FxU32 dst_x, FxU32 dst_y, GrLfbSrcFmt_t src_format, 
										 FxU32 src_width, FxU32 src_height, FxBool pixelPipeline, FxI32 src_stride, void *src_data ) {

	return FXTRUE;
}

FX_ENTRY FxBool FX_CALL grLfbReadRegion( GrBuffer_t src_buffer, FxU32 src_x, FxU32 src_y, FxU32 src_width, FxU32 src_height, FxU32 dst_stride, void *dst_data ) {

	return FXTRUE;
}



/* Important */
FX_ENTRY void FX_CALL grDepthBufferMode( GrDepthBufferMode_t mode ) {
	vlog("Called grDepthBufferMode: mode=" << mode);

	Voodoo.dbMode = mode;

	switch (Voodoo.dbMode) {
	case GR_DEPTHBUFFER_DISABLE:
		_lpD3DD->SetRenderState(D3DRENDERSTATE_ZENABLE, D3DZB_FALSE);
	case GR_DEPTHBUFFER_ZBUFFER:
	case GR_DEPTHBUFFER_ZBUFFER_COMPARE_TO_BIAS:
		_lpD3DD->SetRenderState(D3DRENDERSTATE_ZENABLE, D3DZB_TRUE);
	case GR_DEPTHBUFFER_WBUFFER:
	case GR_DEPTHBUFFER_WBUFFER_COMPARE_TO_BIAS:
		_lpD3DD->SetRenderState(D3DRENDERSTATE_ZENABLE, D3DZB_TRUE);
//		_lpD3DD->SetRenderState(D3DRENDERSTATE_ZENABLE, D3DZB_USEW);
	}

	_lpD3DD->SetRenderState(D3DRENDERSTATE_ZVISIBLE, TRUE);

}


/* Important */
FX_ENTRY void FX_CALL grBufferClear( GrColor_t color, GrAlpha_t alpha, FxU16 depth ) {
	vlog("Called grBufferClear");

	D3DRECT r = {0, 0, Voodoo.width, Voodoo.height};

	// Todo: convert depth buffer field
	_lpD3DViewport->Clear2(1, &r, D3DCLEAR_TARGET | D3DCLEAR_ZBUFFER, 0x0000, 1.0, 0x0000);
}


/* Maybe important */
FX_ENTRY int FX_CALL grBufferNumPending( void ) {
	vlog("Called grBufferNumPending");

	return 0;
}

