// Copyright (C) 1998-1999 Scott Cutler
// Please see the "readme.txt" file for license details

#include "include.h"

/* Maybe important */
FX_ENTRY void FX_CALL grChromakeyValue( GrColor_t value ) {
	vlog("Called grChromakeyValue: color=" << value)

//	COLORREF c = colorconv(value);

	DDCOLORKEY ck;
	ck.dwColorSpaceLowValue = colorconv2(value);
	ck.dwColorSpaceHighValue = colorconv2(value);

	_lpDDSRender->SetColorKey(DDCKEY_SRCBLT, &ck);

}


/* Finished */
FX_ENTRY void FX_CALL grChromakeyMode( GrChromakeyMode_t mode ) {
	vlog("Called grChromakeyMode: mode=" << mode);
	
	if(mode == GR_CHROMAKEY_ENABLE)
		_lpD3DD->SetRenderState(D3DRENDERSTATE_COLORKEYENABLE, TRUE);
	else if (mode == GR_CHROMAKEY_DISABLE)
		_lpD3DD->SetRenderState(D3DRENDERSTATE_COLORKEYENABLE, FALSE);
}


/* Maybe important */
FX_ENTRY void FX_CALL grAlphaTestReferenceValue( GrAlpha_t value ) {
	vlog("Called grAlphaTestReferenceValue: value=" << (int)value);

//	_alphaRef = value;
	_lpD3DD->SetRenderState(D3DRENDERSTATE_ALPHATESTENABLE, TRUE);
	_lpD3DD->SetRenderState(D3DRENDERSTATE_ALPHAREF, (DWORD)value);
}


/* Not important */
FX_ENTRY void FX_CALL grAlphaBlendFunction(
                     GrAlphaBlendFnc_t rgb_sf,   GrAlphaBlendFnc_t rgb_df,
                     GrAlphaBlendFnc_t alpha_sf, GrAlphaBlendFnc_t alpha_df
                     ) {
	vlog("Called grAlphaBlendFunction: rgb_sf=" << rgb_sf << " rgb_df=" << rgb_df << " alpha_sf=" << alpha_sf << " alpha_df=" << alpha_df);


	switch (rgb_sf) {
	case GR_BLEND_ZERO:					_lpD3DD->SetRenderState(D3DRENDERSTATE_SRCBLEND, D3DBLEND_ZERO); break;
	case GR_BLEND_ONE:					_lpD3DD->SetRenderState(D3DRENDERSTATE_SRCBLEND, D3DBLEND_ONE); break;
	case GR_BLEND_DST_COLOR:			_lpD3DD->SetRenderState(D3DRENDERSTATE_SRCBLEND, D3DBLEND_DESTCOLOR); break;
	case GR_BLEND_ONE_MINUS_DST_COLOR:	_lpD3DD->SetRenderState(D3DRENDERSTATE_SRCBLEND, D3DBLEND_INVDESTCOLOR); break;
	case GR_BLEND_SRC_ALPHA:			_lpD3DD->SetRenderState(D3DRENDERSTATE_SRCBLEND, D3DBLEND_SRCALPHA); break;
	case GR_BLEND_ONE_MINUS_SRC_ALPHA:	_lpD3DD->SetRenderState(D3DRENDERSTATE_SRCBLEND, D3DBLEND_INVSRCALPHA); break;
	case GR_BLEND_DST_ALPHA:			_lpD3DD->SetRenderState(D3DRENDERSTATE_SRCBLEND, D3DBLEND_DESTALPHA); break;
	case GR_BLEND_ONE_MINUS_DST_ALPHA:	_lpD3DD->SetRenderState(D3DRENDERSTATE_SRCBLEND, D3DBLEND_INVDESTALPHA); break;
	case GR_BLEND_ALPHA_SATURATE:		_lpD3DD->SetRenderState(D3DRENDERSTATE_SRCBLEND, D3DBLEND_INVSRCALPHA);  break;
//	case GR_BLEND_ALPHA_SATURATE:		_lpD3DD->SetRenderState(D3DRENDERSTATE_SRCBLEND, D3DBLEND_SRCALPHA);  break;
	}

	switch (rgb_df) {
	case GR_BLEND_ZERO:					_lpD3DD->SetRenderState(D3DRENDERSTATE_DESTBLEND, D3DBLEND_ZERO); break;
	case GR_BLEND_ONE:					_lpD3DD->SetRenderState(D3DRENDERSTATE_DESTBLEND, D3DBLEND_ONE); break;
	case GR_BLEND_SRC_COLOR:			_lpD3DD->SetRenderState(D3DRENDERSTATE_DESTBLEND, D3DBLEND_SRCCOLOR); break;
	case GR_BLEND_ONE_MINUS_SRC_COLOR:	_lpD3DD->SetRenderState(D3DRENDERSTATE_DESTBLEND, D3DBLEND_INVSRCCOLOR); break;
	case GR_BLEND_SRC_ALPHA:			_lpD3DD->SetRenderState(D3DRENDERSTATE_DESTBLEND, D3DBLEND_SRCALPHA); break;
	case GR_BLEND_ONE_MINUS_SRC_ALPHA:	_lpD3DD->SetRenderState(D3DRENDERSTATE_DESTBLEND, D3DBLEND_INVSRCALPHA); break;
	case GR_BLEND_DST_ALPHA:			_lpD3DD->SetRenderState(D3DRENDERSTATE_DESTBLEND, D3DBLEND_DESTALPHA); break;
	case GR_BLEND_ONE_MINUS_DST_ALPHA:	_lpD3DD->SetRenderState(D3DRENDERSTATE_DESTBLEND, D3DBLEND_INVDESTALPHA); break;
	case GR_BLEND_PREFOG_COLOR:			_lpD3DD->SetRenderState(D3DRENDERSTATE_DESTBLEND, D3DBLEND_DESTCOLOR); break;
	}
	
	// D3DBLEND_SRCCOLOR
}


/* Maybe important */
FX_ENTRY void FX_CALL grAlphaTestFunction( GrCmpFnc_t function ) {
	vlog("Called grAlphaTestFunction");

	DWORD d3dfunc;

	switch (function) {
	case GR_CMP_NEVER:		d3dfunc = D3DCMP_NEVER;			break;
	case GR_CMP_LESS:		d3dfunc = D3DCMP_LESS;			break;
	case GR_CMP_EQUAL:		d3dfunc = D3DCMP_EQUAL;			break;
	case GR_CMP_LEQUAL:		d3dfunc = D3DCMP_LESSEQUAL;		break;
	case GR_CMP_GREATER:	d3dfunc = D3DCMP_GREATER;		break;
	case GR_CMP_NOTEQUAL:	d3dfunc = D3DCMP_NOTEQUAL;		break;
	case GR_CMP_GEQUAL:		d3dfunc = D3DCMP_GREATEREQUAL;	break;
	case GR_CMP_ALWAYS:		d3dfunc = D3DCMP_ALWAYS;		break;
	}

//	if (_lpD3DD->SetRenderState(D3DRENDERSTATE_ALPHATESTENABLE, TRUE ) != D3D_OK) log("Error setting alpha test function");
	if (_lpD3DD->SetRenderState(D3DRENDERSTATE_ALPHAFUNC, d3dfunc ) != D3D_OK) log("Error setting alpha test function");

}


/* Maybe important */
FX_ENTRY void FX_CALL grColorMask( FxBool rgb, FxBool a ) {
	vlog("Called grColorMask: value=" << rgb);

	DWORD mask;

	if (rgb == FXTRUE) mask |= 0x00FFFFFF;
	if (a == FXTRUE) mask |= 0xFF000000;

	_lpD3DD->SetRenderState(D3DRENDERSTATE_PLANEMASK, mask);

}


/* Maybe important */
FX_ENTRY void FX_CALL grAlphaCombine(GrCombineFunction_t function, GrCombineFactor_t factor,
               GrCombineLocal_t local, GrCombineOther_t other,
               FxBool invert
               ) {
	vlog("Called grAlphaCombine");

}


/* Not important */
FX_ENTRY void FX_CALL grConstantColorValue( GrColor_t value ) {
	vlog("Called grConstantColorValue");

	Voodoo.constantColor = colorconv2(value);
	Voodoo.CCA = ((value & 0xFF000000) >> 24) / 255.0f;
	Voodoo.CCR = ((value & 0x00FF0000) >> 16) / 255.0f;
	Voodoo.CCG = ((value & 0x0000FF00) >> 8) / 255.0f;
	Voodoo.CCB = (value & 0x000000FF) / 255.0f;
}




/* Maybe important */
FX_ENTRY void FX_CALL grColorCombine(
               GrCombineFunction_t function, GrCombineFactor_t factor,
               GrCombineLocal_t local, GrCombineOther_t other,
               FxBool invert ) {
	vlog("Called grColorCombine: func=" << function << "  fac=" << factor << "  local=" << local << "  other=" << other);

	
	// Ok, get ready for massive nastiness :-).
	switch (function) {
	case GR_COMBINE_FUNCTION_LOCAL:
		switch (factor) {
		case GR_COMBINE_FACTOR_ONE:
			switch (local) {
			case GR_COMBINE_LOCAL_ITERATED:
				getcolor = iterated;
				_lpD3DD->SetTextureStageState(0, D3DTSS_COLOROP, D3DTOP_DISABLE);
				break;
			case GR_COMBINE_LOCAL_CONSTANT:
				getcolor = constant;
				_lpD3DD->SetTextureStageState(0, D3DTSS_COLOROP, D3DTOP_DISABLE);
				break;
			default:
				log("  Error: unsupported color combine mode, line=" << __LINE__);
				break;
			}
			break;
		default:
			log("  Error: unsupported color combine mode, line=" << __LINE__);
			break;
		}
		break;
	case GR_COMBINE_FUNCTION_SCALE_OTHER:
		switch (factor) {
		case GR_COMBINE_FACTOR_ZERO:
			getcolor = black;
			_lpD3DD->SetTextureStageState(0, D3DTSS_COLOROP, D3DTOP_DISABLE);
			break;
		case GR_COMBINE_FACTOR_LOCAL:
			switch (local) {
			case GR_COMBINE_LOCAL_ITERATED:
				switch (other) {
				case GR_COMBINE_OTHER_ITERATED:
					getcolor = iterated2;
					_lpD3DD->SetTextureStageState(0, D3DTSS_COLOROP, D3DTOP_DISABLE);
					break;
				case GR_COMBINE_OTHER_CONSTANT:
					getcolor = iterconst;
					_lpD3DD->SetTextureStageState(0, D3DTSS_COLOROP, D3DTOP_DISABLE);
					break;
				case GR_COMBINE_OTHER_TEXTURE:
					getcolor = iterated;
					_lpD3DD->SetTextureStageState(0, D3DTSS_COLOROP, D3DTOP_MODULATE);
					break;
				default:
					log("  Error: unsupported color combine mode, line=" << __LINE__);
					break;
				}
				break;
			case GR_COMBINE_LOCAL_CONSTANT:
				switch (other) {
				case GR_COMBINE_OTHER_ITERATED:
					getcolor = iterconst;
					_lpD3DD->SetTextureStageState(0, D3DTSS_COLOROP, D3DTOP_DISABLE);
					break;
				case GR_COMBINE_OTHER_TEXTURE:
					getcolor = constant;
					_lpD3DD->SetTextureStageState(0, D3DTSS_COLOROP, D3DTOP_MODULATE);
					break;
				case GR_COMBINE_OTHER_CONSTANT:
					getcolor = constant2;
					_lpD3DD->SetTextureStageState(0, D3DTSS_COLOROP, D3DTOP_DISABLE);
					break;
				default:
					log("  Error: unsupported color combine mode, line=" << __LINE__);
					break;
				}
				break;
			default:
				log("  Error: unsupported color combine mode, line=" << __LINE__);
				break;
			}
			break;
		case GR_COMBINE_FACTOR_ONE:
			switch (other) {
			case GR_COMBINE_OTHER_ITERATED:
				getcolor = iterated;
				_lpD3DD->SetTextureStageState(0, D3DTSS_COLOROP, D3DTOP_DISABLE);
				break;
			case GR_COMBINE_OTHER_TEXTURE:
				getcolor = white;
				_lpD3DD->SetTextureStageState(0, D3DTSS_COLOROP, D3DTOP_MODULATE);
				break;
			case GR_COMBINE_OTHER_CONSTANT:
				getcolor = constant;
				_lpD3DD->SetTextureStageState(0, D3DTSS_COLOROP, D3DTOP_DISABLE);
				break;
			default:
				log("  Error: unsupported color combine mode, line=" << __LINE__);
				break;
			}
			break;
		default:
			log("  Error: unsupported color combine mode, line=" << __LINE__);
			break;
		}
		break;
	case GR_COMBINE_FUNCTION_SCALE_OTHER_MINUS_LOCAL_ADD_LOCAL:
			getcolor = iterated;
			_lpD3DD->SetTextureStageState(0, D3DTSS_COLOROP, D3DTOP_DISABLE);
			break;
	default:
		log("  Error: unsupported color combine mode, line=" << __LINE__);
		break;
	}



}


/* Important */
FX_ENTRY void FX_CALL guAlphaSource( GrAlphaSource_t mode ) {
	vlog("Called guAlphaSource: mode=" << mode);
/*
#define D3DTA_SELECTMASK        0x0000000f  // mask for arg selector
#define D3DTA_DIFFUSE           0x00000000  // select diffuse color
#define D3DTA_CURRENT           0x00000001  // select result of previous stage
#define D3DTA_TEXTURE           0x00000002  // select texture color
#define D3DTA_TFACTOR           0x00000003  // select RENDERSTATE_TEXTUREFACTOR

#define D3DTA_COMPLEMENT        0x00000010  // take 1.0 - x
#define D3DTA_ALPHAREPLICATE    0x00000020  // replicate alpha to color components
*/
	// D3DTOP_SELECTARG1
	switch (mode) {
	case GR_ALPHASOURCE_CC_ALPHA:
//		RENDERSTATE_TEXTUREFACTOR
//		_lpD3DD->SetTextureStageState(0, D3DTSS_ALPHAOP, D3DTA_TEXTURE
		break;
	case GR_ALPHASOURCE_ITERATED_ALPHA:
		_lpD3DD->SetTextureStageState(0, D3DTSS_ALPHAOP, D3DTOP_SELECTARG1);
		_lpD3DD->SetTextureStageState(0, D3DTSS_ALPHAARG1, D3DTA_DIFFUSE);
		break;
	case GR_ALPHASOURCE_TEXTURE_ALPHA:
		_lpD3DD->SetTextureStageState(0, D3DTSS_ALPHAOP, D3DTOP_SELECTARG1);
		_lpD3DD->SetTextureStageState(0, D3DTSS_ALPHAARG1, D3DTA_TEXTURE);
		break;
	case GR_ALPHASOURCE_TEXTURE_ALPHA_TIMES_ITERATED_ALPHA:
		_lpD3DD->SetTextureStageState(0, D3DTSS_ALPHAOP, D3DTOP_MODULATE);
		_lpD3DD->SetTextureStageState(0, D3DTSS_ALPHAARG1, D3DTA_TEXTURE);
		_lpD3DD->SetTextureStageState(0, D3DTSS_ALPHAARG2, D3DTA_DIFFUSE);
		break;
	}
//	getcolor = white;
//	_lpD3DD->SetTextureStageState(0, D3DTSS_COLOROP, D3DTOP_DISABLE);

}


/* Important */
FX_ENTRY void FX_CALL guColorCombineFunction( GrColorCombineFnc_t fnc ) {
	vlog("Called guColorCombineFunction: fnc=" << fnc);

//	return;

	switch (fnc) {
//	case GR_COLORCOMBINE_ZERO:							break;
	case GR_COLORCOMBINE_ITRGB:							grColorCombine(GR_COMBINE_FUNCTION_LOCAL, GR_COMBINE_FACTOR_ONE, GR_COMBINE_LOCAL_ITERATED, GR_COMBINE_OTHER_NONE, FXFALSE); break;
//	case GR_COLORCOMBINE_ITRGB:							grColorCombine(GR_COMBINE_FACTOR_ONE, GR_COMBINE_FACTOR_ONE, GR_COMBINE_LOCAL_NONE, GR_COMBINE_OTHER_CONSTANT, FXFALSE); break;
//	case GR_COLORCOMBINE_ITRGB_DELTA0:					break;
	case GR_COLORCOMBINE_DECAL_TEXTURE:					grColorCombine(GR_COMBINE_FUNCTION_SCALE_OTHER, GR_COMBINE_FACTOR_ONE, GR_COMBINE_LOCAL_NONE, GR_COMBINE_OTHER_TEXTURE, FXFALSE); break;
//	case GR_COLORCOMBINE_TEXTURE_TIMES_CCRGB:			break;
	case GR_COLORCOMBINE_TEXTURE_TIMES_ITRGB:			grColorCombine(GR_COMBINE_FUNCTION_SCALE_OTHER, GR_COMBINE_FACTOR_LOCAL, GR_COMBINE_LOCAL_ITERATED, GR_COMBINE_OTHER_TEXTURE, FXFALSE); break;
//	case GR_COLORCOMBINE_TEXTURE_TIMES_ITRGB_ADD_ALPHA:	break;
//	case GR_COLORCOMBINE_TEXTURE_TIMES_ALPHA:			break;
//	case GR_COLORCOMBINE_TEXTURE_ADD_ITRGB:				break;
//	case GR_COLORCOMBINE_TEXTURE_SUB_ITRGB:				break;
//	case GR_COLORCOMBINE_CCRGB:							break;
//	case GR_COLORCOMBINE_CCRGB_BLEND_ITRGB_ON_TEXALPHA:	break;
//	case GR_COLORCOMBINE_DIFF_SPEC_A:					break;
//	case GR_COLORCOMBINE_DIFF_SPEC_B:					break;
//	case GR_COLORCOMBINE_ONE:							break;
	default:											grColorCombine(GR_COMBINE_FUNCTION_SCALE_OTHER, GR_COMBINE_FACTOR_ZERO, GR_COMBINE_LOCAL_NONE, GR_COMBINE_OTHER_NONE, FXFALSE); break;
//	default:											break;
	}

}
