// Copyright (C) 1998-1999 Scott Cutler
// Please see the "readme.txt" file for license details

#include "include.h"



Tex::Tex(const FxU32 address, const GrTexInfo & texinfo) {
	DDSURFACEDESC2 DDSDesc;
	int width, height;
	int bpp;				// Bytes per pixel

	vlog("Called Tex::Tex");
/*
    GrLOD_t           smallLod;
    GrLOD_t           largeLod;
    GrAspectRatio_t   aspectRatio;
    GrTextureFormat_t format;
    void              *data;
*/
	_base = address;
	_aspect = texinfo.aspectRatio;
	_len = TexMemCalc(GR_MIPMAPLEVELMASK_BOTH, texinfo.smallLod, texinfo.largeLod, texinfo.aspectRatio, texinfo.format);

//	if ((_aspect<GR_ASPECT_8x1)||(_aspect>GR_ASPECT_1x8)) _aspect = GR_ASPECT_1x1;
//	if ((_aspect<GR_ASPECT_8x1)||(_aspect>GR_ASPECT_1x8)) log("  Error in aspect ratio in Tex::Tex: " << _aspect);

	
	switch (texinfo.largeLod) {
	case GR_LOD_256:	width=256; height=256; break;
	case GR_LOD_128:	width=128; height=128; break;
	case GR_LOD_64:		width=64; height=64; break;
	case GR_LOD_32:		width=32; height=32; break;
	case GR_LOD_16:		width=16; height=16; break;
	case GR_LOD_8:		width=8; height=8; break;
	case GR_LOD_4:		width=4; height=4; break;
	case GR_LOD_2:		width=2; height=2; break;
	case GR_LOD_1:		width=1; height=1; break;
	}

	switch(texinfo.aspectRatio)	{
	case GR_ASPECT_8x1:	
		height >>= 3; 
		break;
	case GR_ASPECT_4x1:	
		height >>= 2; 
		break;
	case GR_ASPECT_2x1:	
		height >>= 1; 
		break;
	case GR_ASPECT_1x1:	
		break;
	case GR_ASPECT_1x2:	
		width >>= 1; 
		break;
	case GR_ASPECT_1x4:	
		width >>= 2; 
		break;
	case GR_ASPECT_1x8:	
		width >>= 3; 
		break;
	default:
		log("  Error in aspect ratio in Tex::Tex: " << _aspect);
		_aspect = GR_ASPECT_1x1;
		break;
	}

	memset(&DDSDesc, 0, sizeof(DDSURFACEDESC2));
	DDSDesc.ddpfPixelFormat.dwSize = sizeof(DDPIXELFORMAT);

	switch (texinfo.format) {
	case GR_TEXFMT_INTENSITY_8:
		vlog("  pixel format: GR_TEXFMT_INTENSITY_8");
		bpp = 1;
		_paletted = 0;
		DDSDesc.ddpfPixelFormat.dwFlags = DDPF_LUMINANCE;
		DDSDesc.ddpfPixelFormat.dwLuminanceBitCount = 8;
		DDSDesc.ddpfPixelFormat.dwLuminanceBitMask = 0xFF;
		break;
//	case GR_TEXFMT_8BIT:
	case GR_TEXFMT_RGB_332:
		vlog("  pixel format: GR_TEXFMT_RGB_332");
		bpp = 1;
		_paletted = 0;
		DDSDesc.ddpfPixelFormat.dwFlags = DDPF_RGB;
		DDSDesc.ddpfPixelFormat.dwRGBBitCount = 8;
		DDSDesc.ddpfPixelFormat.dwRBitMask = 0xE0;
		DDSDesc.ddpfPixelFormat.dwGBitMask = 0x1C;
		DDSDesc.ddpfPixelFormat.dwBBitMask = 0x03;
		break;
//	case GR_TEXFMT_YIQ_422:            
	case GR_TEXFMT_ALPHA_8:
		vlog("  pixel format: GR_TEXFMT_ALPHA_8");
		bpp = 1;
		_paletted = 0;
		DDSDesc.ddpfPixelFormat.dwFlags = DDPF_ALPHA;
		DDSDesc.ddpfPixelFormat.dwAlphaBitDepth = 8;
		break;
	case GR_TEXFMT_ALPHA_INTENSITY_44:
		vlog("  pixel format: GR_TEXFMT_ALPHA_INTENSITY_44");
		bpp = 1;
		_paletted = 0;
		DDSDesc.ddpfPixelFormat.dwFlags = DDPF_LUMINANCE | DDPF_ALPHAPIXELS;
		DDSDesc.ddpfPixelFormat.dwLuminanceBitCount = 8;
		DDSDesc.ddpfPixelFormat.dwLuminanceAlphaBitMask = 0xF0;
		DDSDesc.ddpfPixelFormat.dwLuminanceBitMask = 0x0F;
		break;
	case GR_TEXFMT_P_8: 
		vlog("  pixel format: GR_TEXFMT_P_8");
		bpp = 1;
		_paletted = 1;
//		DDSDesc.ddpfPixelFormat.dwFlags = DDPF_PALETTEINDEXED8;
		DDSDesc.ddpfPixelFormat.dwFlags = DDPF_RGB | DDPF_PALETTEINDEXED8;
		DDSDesc.ddpfPixelFormat.dwRGBBitCount = 8;
		DDSDesc.ddpfPixelFormat.dwRBitMask = 0x00;
		DDSDesc.ddpfPixelFormat.dwGBitMask = 0x00;
		DDSDesc.ddpfPixelFormat.dwBBitMask = 0x00;
		DDSDesc.ddpfPixelFormat.dwRGBAlphaBitMask = 0x00;
		break;
//	case GR_TEXFMT_16BIT:
	case GR_TEXFMT_ARGB_8332:
		vlog("  pixel format: GR_TEXFMT_ARGB_8332");
		bpp = 2;
		_paletted = 0;
		DDSDesc.ddpfPixelFormat.dwFlags = DDPF_RGB | DDPF_ALPHAPIXELS;
		DDSDesc.ddpfPixelFormat.dwRGBBitCount = 16;
		DDSDesc.ddpfPixelFormat.dwRGBAlphaBitMask = 0xFF00;
		DDSDesc.ddpfPixelFormat.dwRBitMask = 0x00E0;
		DDSDesc.ddpfPixelFormat.dwGBitMask = 0x001C;
		DDSDesc.ddpfPixelFormat.dwBBitMask = 0x0003;
		break;
//	case GR_TEXFMT_AYIQ_8422:
	case GR_TEXFMT_RGB_565:
		vlog("  pixel format: GR_TEXFMT_RGB_565");
		bpp = 2;
		_paletted = 0;
		DDSDesc.ddpfPixelFormat.dwFlags = DDPF_RGB;
		DDSDesc.ddpfPixelFormat.dwRGBBitCount = 16;
		DDSDesc.ddpfPixelFormat.dwRBitMask = 0xF800;
		DDSDesc.ddpfPixelFormat.dwGBitMask = 0x07E0;
		DDSDesc.ddpfPixelFormat.dwBBitMask = 0x001F;
		DDSDesc.ddpfPixelFormat.dwRGBAlphaBitMask = 0x0000;
		break;
	case GR_TEXFMT_ARGB_1555:
		vlog("  pixel format: GR_TEXFMT_ARGB_1555");
		bpp = 2;
		_paletted = 0;
		DDSDesc.ddpfPixelFormat.dwFlags = DDPF_RGB | DDPF_ALPHAPIXELS;
		DDSDesc.ddpfPixelFormat.dwRGBBitCount = 16;
		DDSDesc.ddpfPixelFormat.dwRGBAlphaBitMask = 0x8000;
		DDSDesc.ddpfPixelFormat.dwRBitMask = 0x7C00;
		DDSDesc.ddpfPixelFormat.dwGBitMask = 0x03E0;
		DDSDesc.ddpfPixelFormat.dwBBitMask = 0x001F;
		break;
	case GR_TEXFMT_ARGB_4444:
		vlog("  pixel format: GR_TEXFMT_ARGB_4444");
		bpp = 2;
		_paletted = 0;
		DDSDesc.ddpfPixelFormat.dwFlags = DDPF_RGB | DDPF_ALPHAPIXELS;
		DDSDesc.ddpfPixelFormat.dwRGBBitCount = 16;
		DDSDesc.ddpfPixelFormat.dwRGBAlphaBitMask = 0xF000;
		DDSDesc.ddpfPixelFormat.dwRBitMask = 0x0F00;
		DDSDesc.ddpfPixelFormat.dwGBitMask = 0x00F0;
		DDSDesc.ddpfPixelFormat.dwBBitMask = 0x000F;
		break;
	case GR_TEXFMT_ALPHA_INTENSITY_88:
		vlog("  pixel format: GR_TEXFMT_ALPHA_INTENSITY_88");
		bpp = 2;
		_paletted = 0;
		DDSDesc.ddpfPixelFormat.dwFlags = DDPF_LUMINANCE | DDPF_ALPHAPIXELS;
		DDSDesc.ddpfPixelFormat.dwLuminanceBitCount = 16;
		DDSDesc.ddpfPixelFormat.dwLuminanceAlphaBitMask = 0xFF00;
		DDSDesc.ddpfPixelFormat.dwLuminanceBitMask = 0x00FF;
		break;
//	case GR_TEXFMT_AP_88:       
	}

	DDSDesc.dwSize = sizeof(DDSURFACEDESC2);
	DDSDesc.dwFlags = DDSD_CAPS | DDSD_WIDTH | DDSD_HEIGHT | DDSD_PIXELFORMAT;
	DDSDesc.ddsCaps.dwCaps = DDSCAPS_3DDEVICE | DDSCAPS_TEXTURE;// | DDSCAPS_VIDEOMEMORY;
	DDSDesc.ddsCaps.dwCaps2 = DDSCAPS2_TEXTUREMANAGE;
	DDSDesc.dwWidth = _texwidth = width;
	DDSDesc.dwHeight = _texheight = height;
	DDSDesc.dwTextureStage = 0;
	DDSDesc.lPitch = width * bpp;
	DDSDesc.dwMipMapCount = texinfo.largeLod - texinfo.smallLod + 1;
//	DDSDesc.dwMipMapCount = 1;

	HRESULT href;

	href = _lpDD->CreateSurface(&DDSDesc, &_texDDS, 0);
	if (href == DD_OK) {
		vlog("  Created surface:");
		vlog("   width=" << width);
		vlog("   height=" << height);
	} else {
		_tex = 0;
		log("Error code " << (href&0xFF) << " in Tex::Tex on CreateSurface, line " << __LINE__ << ", file " << __FILE__);
	}


	// Ok, now we copy the texture to the surface
	DDSURFACEDESC2 DDSD;
	RECT r = {0, 0, width, height};

	DDSD.dwSize = sizeof(DDSURFACEDESC2);

	if (_texDDS->Lock(&r, &DDSD, DDLOCK_WAIT, 0) == DD_OK) {

		vlog("   width=" << DDSD.dwWidth);
		vlog("   height=" << DDSD.dwHeight);
		vlog("   pitch=" << DDSD.lPitch);
		for (unsigned int i=0; i<DDSD.dwHeight; i++) memcpy((char *)DDSD.lpSurface+(i*DDSD.lPitch), (char *)texinfo.data+(i*DDSD.dwWidth*bpp), DDSD.dwWidth*bpp);
		
		if (_texDDS->Unlock(&r) != DD_OK) log("Could not unlock surface on line " << __LINE__ << ", file " << __FILE__);

	} else {
		_tex = 0;
		log("Could not lock texture surface on line " << __LINE__);
		return;
	}

	if (_paletted) {
			LPDIRECTDRAWPALETTE DDP;
		    PALETTEENTRY palette[256];

		    memset(palette, 0, sizeof(palette));
			if (_lpDD->CreatePalette(DDPCAPS_8BIT | DDPCAPS_ALLOW256, palette, &DDP, 0) != DD_OK) {
				_tex = 0;
				log("Error creating palette");
			}
			if (_texDDS->SetPalette(DDP) != DD_OK) {
				_tex = 0;
				log("Error setting palette");
			}
			DDP->Release();
	}



	if (_texDDS->QueryInterface(IID_IDirect3DTexture2, (void**)&_tex) != D3D_OK) {
		_tex = 0;
		log("Error querying interface");
	}

//	if (texinfo.format == GR_TEXFMT_RGB_565) _tex = 0;

}

Tex::~Tex() {
	vlog("Called Tex::~Tex");
	_tex->Release();
	_texDDS->Release();
}


FxU32 intersect(const Tex &t1, const Tex &t2) {
	vvlog("Called Tex::intersect");

	if (t1.base() < t2.base()) {
		if ((t1.base()+t1.len())>t2.base()) 
			return FXTRUE;
		else 
			return FXFALSE;
	} else if (t1.base() > t2.base()) {
		if ((t2.base()+t2.len())>t1.base()) 
			return FXTRUE;
		else 
			return FXFALSE;
	} else {
		return FXTRUE;
	}
}

void Tex::setpalette() const {
	LPDIRECTDRAWPALETTE DDP;

	if (_texDDS->GetPalette(&DDP) != DD_OK) log("Error getting palette");
	if (DDP->SetEntries(0, 0, 256, Voodoo.palette) != DD_OK) log("Error setting palette");
	DDP->Release();
}
