// Copyright (C) 1998-1999 Scott Cutler
// Please see the "readme.txt" file for license details

#include "include.h"

/* Maybe important */
FX_ENTRY void FX_CALL grTexDownloadTable( GrChipID_t tmu, GrTexTable_t type, void *data ) {
	vlog("Called grTexDownloadTable");

	int i;

	switch (type) {
	case GR_TEXTABLE_PALETTE:
		for (i=0; i<256; i++) {
			Voodoo.palette[i].peRed = (((FxU32 *)data)[i] >> 16) & 0xFF;
			Voodoo.palette[i].peGreen = (((FxU32 *)data)[i] >> 8) & 0xFF;
			Voodoo.palette[i].peBlue = (((FxU32 *)data)[i]) & 0xFF;
//			Voodoo.palette[i].peRed = i;
//			Voodoo.palette[i].peGreen = i;
//			Voodoo.palette[i].peBlue = i;
			Voodoo.palette[i].peFlags = 0;
		}
		break;
	case GR_TEXTABLE_NCC0:
		log("Error: NCC textures not supported.\n");
		break;
	case GR_TEXTABLE_NCC1:
		log("Error: NCC textures not supported.\n");
		break;
	}


}

FxU32 TexMemCalc(FxU32 evenOdd, GrLOD_t lodmin, GrLOD_t lodmax,
                 GrAspectRatio_t aspect, GrTextureFormat_t fmt)
{
	int mem=0, inc, min;

	if (evenOdd==GR_MIPMAPLEVELMASK_BOTH) {
		min = lodmin;
		inc = 1;
	} else if (evenOdd==GR_MIPMAPLEVELMASK_EVEN) {
		min = (lodmin+1)&0xFFFFFFFE;
		inc = 2;
	} else {
		min = (lodmin&0xFFFFFFFE)+1;
		inc = 2;
	}


	for (int i=min; i<=lodmax; i+=inc) {
		switch (i) {
			case GR_LOD_256:	mem += 256*256; break;
			case GR_LOD_128:	mem += 128*128; break;
			case GR_LOD_64:		mem += 64*64; break;
			case GR_LOD_32:		mem += 32*32; break;
			case GR_LOD_16:		mem += 16*16; break;
			case GR_LOD_8:		mem += 8*8; break;
			case GR_LOD_4:		mem += 4*4; break;
			case GR_LOD_2:		mem += 2*2; break;
			case GR_LOD_1:		mem += 1*1; break;
		}
	}

	switch(aspect)	{
		case GR_ASPECT_8x1:	mem >>= 3; break;
		case GR_ASPECT_4x1:	mem >>= 2; break;
		case GR_ASPECT_2x1:	mem >>= 1; break;
		case GR_ASPECT_1x1:	break;
		case GR_ASPECT_1x2:	mem >>= 1; break;
		case GR_ASPECT_1x4:	mem >>= 2; break;
		case GR_ASPECT_1x8:	mem >>= 3; break;
	}

	switch(fmt)
	{
		case GR_TEXFMT_RGB_332:
		case GR_TEXFMT_YIQ_422:
		case GR_TEXFMT_ALPHA_8:
		case GR_TEXFMT_INTENSITY_8:
		case GR_TEXFMT_ALPHA_INTENSITY_44:
		case GR_TEXFMT_P_8:					break;
		case GR_TEXFMT_RGB_565:
		case GR_TEXFMT_ARGB_8332:
		case GR_TEXFMT_AYIQ_8422:
		case GR_TEXFMT_ARGB_1555:
		case GR_TEXFMT_ARGB_4444:
		case GR_TEXFMT_ALPHA_INTENSITY_88:
		case GR_TEXFMT_AP_88:				mem <<= 1;  break;
	}

	return (mem+7)%8;
}


/* Maybe important */
FX_ENTRY FxU32 FX_CALL grTexCalcMemRequired(
                     GrLOD_t lodmin, GrLOD_t lodmax,
                     GrAspectRatio_t aspect, GrTextureFormat_t fmt) {
	vlog("Called grTexCalcMemRequired");

	return TexMemCalc(GR_MIPMAPLEVELMASK_BOTH, lodmin, lodmax, aspect, fmt);
}


/* Maybe important */
FX_ENTRY FxU32 FX_CALL 
grTexTextureMemRequired( FxU32     evenOdd,
						GrTexInfo *info   ) {
	vlog("Called grTexTextureMemRequired");

	return TexMemCalc(GR_MIPMAPLEVELMASK_BOTH, info->smallLod, info->largeLod, info->aspectRatio, info->format);
//	return TexMemCalc(evenOdd, info->smallLod, info->largeLod, info->aspectRatio, info->format);
}


/* Important */
FX_ENTRY void FX_CALL grTexDownloadMipMap( GrChipID_t tmu,
                     FxU32      startAddress,
                     FxU32      evenOdd,
                     GrTexInfo  *info ) {
	vlog("Called grTexDownloadMipMap");

	_texlist.AddTex(startAddress, *info);

}


/* Maybe important */
FX_ENTRY void FX_CALL grTexDownloadMipMapLevel( GrChipID_t        tmu,
                          FxU32             startAddress,
                          GrLOD_t           thisLod,
                          GrLOD_t           largeLod,
                          GrAspectRatio_t   aspectRatio,
                          GrTextureFormat_t format,
                          FxU32             evenOdd,
                          void              *data ) {
	vlog("Called grTexDownloadMipMapLevel");

	GrTexInfo info;

	if (thisLod != largeLod) return;

	info.smallLod = thisLod;
	info.largeLod = thisLod;
	info.aspectRatio = aspectRatio;
	info.format = format;
	info.data = data;
	_texlist.AddTex(startAddress, info);


}

/* Maybe important */
FX_ENTRY void FX_CALL grTexDownloadMipMapLevelPartial( GrChipID_t        tmu,
                                 FxU32             startAddress,
                                 GrLOD_t           thisLod,
                                 GrLOD_t           largeLod,
                                 GrAspectRatio_t   aspectRatio,
                                 GrTextureFormat_t format,
                                 FxU32             evenOdd,
                                 void              *data,
                                 int               start,
                                 int               end ) {
	vlog("Called grTexDownloadMipMapLevelPartial");

}

/* Maybe important */
FX_ENTRY FxU32 FX_CALL grTexMinAddress( GrChipID_t tmu ) {
	vlog("Called grTexMinAddress: tmu=" << tmu << "  address=" << int(_tmuMem));

	return (FxU32)_tmuMem;
}


/* Maybe important */
FX_ENTRY FxU32 FX_CALL grTexMaxAddress( GrChipID_t tmu ) {
	vlog("Called grTexMaxAddress: tmu=" << tmu << "  address=" << int(_tmuMem+_tmuRamBytes));

	return (FxU32)(_tmuMem+_tmuRamBytes);
}


/* Not important */
FX_ENTRY void FX_CALL grTexLodBiasValue(GrChipID_t tmu, float bias ) {
	vlog("Called grTexLocalBiasValue");
	vlog("  tmu=" << tmu);
	vlog("  bias=" << bias);

}

/* Not important */
FX_ENTRY void FX_CALL grTexClampMode(
               GrChipID_t tmu,
               GrTextureClampMode_t s_clampmode,
               GrTextureClampMode_t t_clampmode
               ) {
	vvlog("Called grTexClampMode");
	vvlog("  tmu=" << tmu);
	vvlog("  s_clampmode=" << s_clampmode);
	vvlog("  t_clampmode=" << t_clampmode);

	_lpD3DD->SetTextureStageState(tmu, D3DTSS_ADDRESSU, s_clampmode == GR_TEXTURECLAMP_WRAP ? D3DTADDRESS_WRAP : D3DTADDRESS_CLAMP);
	_lpD3DD->SetTextureStageState(tmu, D3DTSS_ADDRESSV, t_clampmode == GR_TEXTURECLAMP_WRAP ? D3DTADDRESS_WRAP : D3DTADDRESS_CLAMP);


}

/* Important */
FX_ENTRY void FX_CALL guTexCombineFunction(
                     GrChipID_t tmu,
                     GrTextureCombineFnc_t fnc
                     ) {
	vlog("Called guTexCombineFunction");
	vlog("  tmu=" << tmu);
	vlog("  func=" << fnc);

}

/* Not important */
FX_ENTRY void FX_CALL grTexMipMapMode( GrChipID_t     tmu, 
                 GrMipMapMode_t mode,
                 FxBool         lodBlend ) {
	vvlog("Called grTexMipMapMode");
	vvlog("  tmu=" << tmu);
	vvlog("  mode=" << mode);
	vvlog("  blend=" << lodBlend);

}


/* Not important */
FX_ENTRY void FX_CALL grTexFilterMode(
                GrChipID_t tmu,
                GrTextureFilterMode_t minfilter_mode,
                GrTextureFilterMode_t magfilter_mode
                ) {
	vvlog("Called grTexFilterMode:");
	vvlog("  tmu=" << tmu);
	vvlog("  minmode=" << minfilter_mode);
	vvlog("  magmode=" << magfilter_mode);

	switch(minfilter_mode) {
	case GR_TEXTUREFILTER_POINT_SAMPLED:
		if (_lpD3DD->SetRenderState(D3DRENDERSTATE_TEXTUREMIN, D3DFILTER_NEAREST) != D3D_OK) log("Error setting texture filter mode in grTexFilterMode.");
		break;
    case GR_TEXTUREFILTER_BILINEAR:
		if (_lpD3DD->SetRenderState(D3DRENDERSTATE_TEXTUREMIN, D3DFILTER_LINEAR) != D3D_OK) log("Error setting texture filter mode in grTexFilterMode.");
		break;
	}

	switch(magfilter_mode) {
	case GR_TEXTUREFILTER_POINT_SAMPLED:
		if (_lpD3DD->SetRenderState(D3DRENDERSTATE_TEXTUREMAG, D3DFILTER_NEAREST) != D3D_OK) log("Error setting texture filter mode in grTexFilterMode.");
		break;
    case GR_TEXTUREFILTER_BILINEAR:
		if (_lpD3DD->SetRenderState(D3DRENDERSTATE_TEXTUREMAG, D3DFILTER_LINEAR) != D3D_OK) log("Error setting texture filter mode in grTexFilterMode.");
		break;
	}
}



/* Important */
FX_ENTRY void FX_CALL grTexSource( GrChipID_t tmu, FxU32 startAddress, FxU32 evenOdd, GrTexInfo *info ) {
	vvlog("Called grTexSource");

	Voodoo.curtex = _texlist.GetTex(startAddress, *info);

	if (Voodoo.curtex) {
		if (_lpD3DD->SetTexture(0, Voodoo.curtex->tex()) != D3D_OK) log("Error setting texture");
	} else {
		if (_lpD3DD->SetTexture(0, 0) != D3D_OK) log("Error setting texture");
//		_lpD3DD->SetTextureStageState(0, D3DTSS_COLOROP, D3DTOP_DISABLE);
		log("Error: couldn't find texture");
	}
}


/* Only for Marble32 */
FX_ENTRY GrMipMapId_t FX_CALL guTexAllocateMemory( 	GrChipID_t tmu,
											FxU8 evenOddMask,
											int width, int height,
											GrTextureFormat_t format,
											GrMipMapMode_t mmMode,
											GrLOD_t smallLod, GrLOD_t largeLod,
											GrAspectRatio_t aspectRatio,
											GrTextureClampMode_t sClampMode,
											GrTextureClampMode_t tClampMode,
											GrTextureFilterMode_t minFilterMode,
											GrTextureFilterMode_t magFilterMode,
											float lodBias,
											FxBool lodBlend ) {
	log("Called guTexAllocateMemory");


	return 0;
}

/* Only for Marble32 */
FX_ENTRY void FX_CALL guTexDownloadMipMap( GrMipMapId_t mmid, const void *src, const GuNccTable *nccTable ) {
	vlog("Called guTexDownloadMipMap");

}

/* Only for Marble32 */
FX_ENTRY void FX_CALL guTexSource( GrMipMapId_t mmid ) {
	vlog("Called grTexSource");

}


/* Only for Marble32 */
FX_ENTRY void FX_CALL grTexCombine( GrChipID_t tmu,
							GrCombineFunction_t rgb_function,
							GrCombineFactor_t rgb_factor,
							GrCombineFunction_t alpha_function,
							GrCombineFactor_t alpha_factor,
							FxBool rgb_invert,
							FxBool alpha_invert ) {
	vlog("Called grTexCombine: (" << rgb_function << ", " << rgb_factor << ", " << alpha_function << ", " << alpha_factor << ", " << rgb_invert << ", " << alpha_invert << ")");
/*
	switch (rgb_function) {
	case GR_COMBINE_FUNCTION_LOCAL:
		switch (rgb_factor) {
		case GR_COMBINE_FACTOR_ZERO:
			_lpD3DD->SetTextureStageState(0, D3DTSS_COLOROP, D3DTOP_DISABLE);
			break;
		}
		break;
	}

	switch (alpha_function) {
	case GR_COMBINE_FUNCTION_LOCAL:
		switch (rgb_factor) {
		case GR_COMBINE_FACTOR_ZERO:
			_lpD3DD->SetTextureStageState(0, D3DTSS_ALPHAOP, D3DTOP_DISABLE);
			break;
		}
		break;
	}
*/
}


FX_ENTRY void FX_CALL grTexCombineFunction(GrChipID_t tmu, GrTextureCombineFnc_t fnc) {
	
}
