// Copyright (C) 1998-1999 Scott Cutler
// Please see the "readme.txt" file for license details

#include "include.h"

/* Maybe important */
FX_ENTRY void FX_CALL grHints(GrHints_t type, FxU32 hintMask) {
	log("Called grHints:");
	log("  type=" << type);
	log("  hintmask=" << hintMask);

}


/* Not important */
FX_ENTRY void FX_CALL grGammaCorrectionValue( float value ) {
	vlog("Called grGammaCorrectionValue: value=" << value);

}


/* Maybe important */
FX_ENTRY void FX_CALL grErrorSetCallback( GrErrorCallbackFnc_t fnc ) {
	log("Called grErrorSetCallback");

	_errorFnc = fnc;
}


/* Not important */
FX_ENTRY void FX_CALL grDitherMode( GrDitherMode_t mode ) {
	vlog("Called grDitherMode: mode=" << mode);

	switch(mode) {
	case GR_DITHER_DISABLE:
		_lpD3DD->SetRenderState(D3DRENDERSTATE_DITHERENABLE, FALSE);
		break;
	case GR_DITHER_2x2:
	case GR_DITHER_4x4:
		_lpD3DD->SetRenderState(D3DRENDERSTATE_DITHERENABLE, TRUE);
		break;
	}

}


/* Only for Marble32 */
FX_ENTRY void FX_CALL grDisableAllEffects() {
	vlog("Called grDisableAllEffects");

	grAlphaBlendFunction(GR_BLEND_ONE, GR_BLEND_ZERO, GR_BLEND_ONE, GR_BLEND_ZERO);
	grAlphaTestFunction(GR_CMP_ALWAYS);
	grChromakeyMode(GR_CHROMAKEY_DISABLE);
	grDepthBufferMode(GR_DEPTHBUFFER_DISABLE);
	grFogMode(GR_FOG_DISABLE);

}


D3DCOLOR colorconv2(GrColor_t c) {
	vlog("  Called colorconv");

	FxU32 r, g, b;

	switch (_color_format) {
	case GR_COLORFORMAT_RGBA: 
		r = (c >> 24);
		g = (c >> 16) & 0xFF;
		b = (c >> 8) & 0xFF;
		break;
	case GR_COLORFORMAT_ARGB:
		r = (c >> 16) & 0xFF;
		g = (c >> 8) & 0xFF;
		b = c & 0xFF;
		break;
	case GR_COLORFORMAT_BGRA:
		r = (c >> 8) & 0xFF;
		g = (c >> 16) & 0xFF;
		b = (c >> 24);
		break;
	case GR_COLORFORMAT_ABGR:
		r = c & 0xFF;
		g = (c >> 8) & 0xFF;
		b = (c >> 16) & 0xFF;
		break;
	}

	return ((r<<16)+(g<<8)+b);
}


D3DCOLOR iterated(float a, float r, float g, float b) {
//	return 0xFFFFFFFF;
	return D3DCOLOR(((int(a)&0xFF) << 24) + ((int(r)&0xFF) << 16) + ((int(g)&0xFF) << 8) + (int(b)&0xFF));
}

D3DCOLOR iterated2(float a, float r, float g, float b) {
	return D3DCOLOR(((int(a)&0xFF) << 24) + ((int(r)&0xFF) << 16) + ((int(g)&0xFF) << 8) + (int(b)&0xFF));
}

D3DCOLOR iterconst(float a, float r, float g, float b) {
	return D3DCOLOR(
		((int(a+Voodoo.CCA)&0xFF) << 24) + 
		((int(r+Voodoo.CCR)&0xFF) << 16) + 
		((int(g+Voodoo.CCG)&0xFF) << 8) + 
		(int(b+Voodoo.CCB)&0xFF));
}

D3DCOLOR constant(float a, float r, float g, float b) {
	return Voodoo.constantColor;
}

D3DCOLOR constant2(float a, float r, float g, float b) {
	return Voodoo.constantColor;
}

D3DCOLOR white(float a, float r, float g, float b) {
	return 0xFFFFFFFF;
}

D3DCOLOR black(float a, float r, float g, float b) {
	return 0x00000000;
}

D3DTLVERTEX vconv(const GrVertex & v) {
//	vlog("  Called vconv");

	D3DTLVERTEX vc;

//	const float vertex_snap = float(3L << 18);

	if( v.x > 2048 ) {
		vc.sx = v.x - (FLOAT)(((DWORD)v.x) & 0xfffff000) - 0.5001f;
		vc.sy = v.y - (FLOAT)(((DWORD)v.y) & 0xfffff000) - 0.5001f;
//		vc.sx = v.x - vertex_snap;
//		vc.sy = v.y - vertex_snap;
	} else {
		vc.sx = v.x - 0.5001f;
		vc.sy = v.y - 0.5001f;
	}

	if(Voodoo.dbMode == GR_DEPTHBUFFER_WBUFFER)
		vc.sz = 1.0f/(v.oow*65536.0f);
	else if(Voodoo.dbMode == GR_DEPTHBUFFER_ZBUFFER)
		vc.sz = v.ooz/65536.0f;


	vc.color = getcolor(v.a, v.r, v.g, v.b);

	if (Voodoo.curtex) {
		int aspect = Voodoo.curtex->aspect();

		float w = 1.0f / v.oow;

		vc.tu = v.tmuvtx[0].sow*smul[aspect]*w;
		vc.tv = v.tmuvtx[0].tow*tmul[aspect]*w;
/*
		if (Voodoo.uclamp) {
			if (vc.tu > 1.0) vc.tu = 1.0;
			if (vc.tu < 0.0) vc.tu = 0.0;
		}
		if (Voodoo.vclamp) {
			if (vc.tv > 1.0) vc.tv = 1.0;
			if (vc.tv < 0.0) vc.tv = 0.0;
		}
		*/
	}

	vc.rhw = v.oow;


	if (Voodoo.origin == GR_ORIGIN_LOWER_LEFT) vc.sy = Voodoo.height - vc.sy;


	return vc;
}
